import requests
import urllib3
import json

from podeli.model.BnplClientInfo import BnplClientInfo
from podeli.model.BnplRequest import BnplRequest
from podeli.model.CancelOrderRequest import CancelOrderRequest
from podeli.model.CancelOrderResponse import CancelOrderResponse
from podeli.model.CommitOrderRequest import CommitOrderRequest
from podeli.model.CommitOrderResponse import CommitOrderResponse
from podeli.model.CreateOrderRequest import CreateOrderRequest
from podeli.model.BnplOrder import *
from podeli.model.CreateOrderResponse import *
from podeli.error import *
from podeli.model.InfoOrderRequest import InfoOrderRequest
from podeli.model.InfoOrderResponse import InfoOrderResponse
from podeli.model.RefundOrderRequest import RefundOrderRequest, RefundInfo
from podeli.model.RefundOrderResponse import RefundOrderResponse


class BnplApi:
    """Класс BNPL API

       Используется для вызова методов сервиса Podeli
    """
    def __init__(
            self,
            login: str,
            password: str,
            cert_file: str,
            cert_key: str,
            url: str = 'https://api-dev.podeli.ru/',
            proxy: str = None,
            verify_ssl: bool = False
    ):
        """
        Создание класса BNPL API
        :param login: логин пользователя
        :param password: пароль пользователя
        :param cert_file: путь к файлу сертификата PEM
        :param cert_key:  путь к файлу ключа сертификата
        :param url: адрес сервиса
        :param proxy: настройки прокси
        :param verify_ssl: признак необходимости проверки сертификата. Необходимо отключать для самоподписанных сертификаторв
        """
        self.BaseUrl = url
        self.login = login
        self.password = password
        self.proxy = proxy
        self.cert_file = cert_file
        self.cert_key = cert_key
        self.verify_ssl = verify_ssl

        # Отключение предупреждений о самоподписанном сертификате при отключенной верификации сертификата
        if not self.verify_ssl:
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    def create_order(self,
                     order: BnplOrder,
                     client: BnplClientInfo,
                     x_correlation_id: str,
                     success_url: str,
                     fail_url: str,
                     notification_url: str = None) -> CreateOrderResponse:
        """
        Метод создания заказа
        :param order: данные о заказе ``podeli.model.BnplOrder``
        :param client: данные о клиенте ``podeli.model.BnplClientInfo``
        :param x_correlation_id: идентификатор заказа на стороне сервиса, строка GUID
        :param success_url: адрес для редиректа клиента в случае успешного создания заказа
        :param fail_url: адрес для редиректа клиента в случае неудачного создания заказа
        :param notification_url: адрес для приема уведомлений
        :return: информация о созданном заказе ``podeli.model.CreateOrderResponse``
        :rtype: podeli.model.CreateOrderResponse
        :raises: ``podeli.error.BnlpStatusError``: если статус ответа не 200 (не удалось создать заказ по какой-то причине)
        :raises: ``podeli.error.BnlpFormatError``: не распознан ответ сервиса
        """
        create_request = CreateOrderRequest(
            order=order,
            client=client,
            x_correlation_id=x_correlation_id,
            success_url=success_url,
            fail_url=fail_url,
            notification_url=notification_url
        )

        response = self.request_api(create_request)
        result = self.__check_response(response)

        return CreateOrderResponse.from_response(result)

    def cancel_order(self, order_id: int, x_correlation_id: str, initiator: str) -> CancelOrderResponse:
        """
        Метод отмены заказа
        :param order_id: идентификатор заказа на стороне клиента
        :param x_correlation_id: идентификатор заказа на стороне сервиса, строка GUID
        :param initiator: инициатор отмены заказа ('shop' или 'client')
        :return: результат отмены заказа ``podeli.model.CancelOrderResponse``
        :raises: ``podeli.error.BnlpStatusError``: если статус ответа не 200 (не удалось создать заказ по какой-то причине)
        :raises: ``podeli.error.BnlpFormatError``: не распознан ответ сервиса
        :raises: ``ValueError`` если инициатор отмены не 'client' или 'shop'
        """
        cancel_request = CancelOrderRequest(
            order_id=order_id,
            x_correlation_id=x_correlation_id,
            initiator=initiator
        )
        response = self.request_api(cancel_request)
        result = self.__check_response(response)
        return CancelOrderResponse.from_response(result)

    def refund_order(self, order_id: int, x_correlation_id: str, refund_info: RefundInfo) -> RefundOrderResponse:
        """
        Метод возврата заказа
        :param order_id: идентификатор заказа на стороне клиента
        :param x_correlation_id: идентификатор заказа на стороне сервиса, строка GUID
        :param refund_info: информация об отмене ``podeli.model.RefundOrderRequest.RefundInfo``
        :return: результат возврата ``podeli.model.RefundOrderResponse``
        :rtype: RefundOrderResponse
        :raises: ``podeli.error.BnlpStatusError``: если статус ответа не 200 (не удалось сделать возврат по какой-то причине)
        :raises: ``podeli.error.BnlpFormatError``: не распознан ответ сервиса
        """
        refund = RefundOrderRequest(order_id=order_id, x_correlation_id=x_correlation_id, refund_info=refund_info)
        response = self.request_api(refund)
        result = self.__check_response(response)
        return RefundOrderResponse.from_response(result)

    def commit_order(self, order: BnplOrder, x_correlation_id: str) -> CommitOrderResponse:
        """
        Метод подтверждения заказа
        :param order: инфромация о заказе ``podeli.model.BnplOrder``
        :param x_correlation_id: идентификатор заказа на стороне сервиса, строка GUID
        :return: результат подтверждения заказа ``podeli.model.CommitOrderResponse``
        :rtype: CommitOrderResponse
        :raises: ``podeli.error.BnlpStatusError``: если статус ответа не 200 (не удалось подтвердить заказ по какой-то причине)
        :raises: ``podeli.error.BnlpFormatError``: не распознан ответ сервиса
        """
        return self.__commit_order_raw(order_id=order.id, x_correlation_id=x_correlation_id, amount=order.amount,
                                       prepaid_amount=order.prepaidAmount, items=order.items)

    def __commit_order_raw(self_, order_id: int, x_correlation_id: str, amount: float, prepaid_amount: float,
                           items: list[BnplOrderItem] = None) -> CommitOrderResponse:
        """
        Метод подтверждения заказа
        :param order_id: идентификатор заказа на стороне клиента
        :param x_correlation_id: идентификатор заказа на стороне сервиса, строка GUID
        :param amount: стоимость заказа в рублях
        :param prepaid_amount: величина предоплаты
        :param items: список товаров в заказе ``podeli.model.BnplOrderItem.BnplOrderItem``
        :return: результат подтверждения заказа ``podeli.model.CommitOrderResponse``
        :rtype: CommitOrderResponse
        :raises: ``podeli.error.BnlpStatusError``: если статус ответа не 200 (не удалось подтвердить заказ по какой-то причине)
        :raises: ``podeli.error.BnlpFormatError``: не распознан ответ сервиса
        """
        commit_request = CommitOrderRequest(
            order_id=order_id,
            x_correlation_id=x_correlation_id,
            amount=amount,
            prepaid_amount=prepaid_amount,
            items=items)
        response = self_.request_api(commit_request)
        result = self_.__check_response(response)
        return CommitOrderResponse.from_response(result)

    def get_order_info(
            self,
            order_id: int,
            x_correlation_id: str,
            ) -> InfoOrderResponse:
        """
        Метод полученя информации о состоянии заказа
        :param order_id: идентификатор заказа на стороне клиента
        :param x_correlation_id: идентификатор заказа на стороне сервиса, строка GUID
        :return: информация о состоянии заказа ``podeli.model.InfoOrderResponse.InfoOrderResponse``
        :rtype: InfoOrderResponse
        :raises: ``podeli.error.BnlpStatusError``: если статус ответа не 200 (не удалось создать заказ по какой-то причине)
        :raises: ``podeli.error.BnlpFormatError``: не распознан ответ сервиса
        """
        info_request = InfoOrderRequest(
            order_id=order_id,
            x_correlation_id=x_correlation_id
        )
        response = self.request_api(info_request)
        result = self.__check_response(response)
        return InfoOrderResponse.from_response(result)

    @staticmethod
    def __check_response(response: requests.Response):
        """
        Служебный метод для проверки ответа сервиса
        :param response: ответ сервиса типа ``requests.Response``
        :return: ответ сервиса - строка JSON
        :raises: ``podeli.error.BnlpStatusError``: если статус ответа не 200 (не удалось создать заказ по какой-то причине)
        """
        if response.status_code == 200:
            return json.loads(response.text)
        else:
            raise BnlpStatusError(message="RequestError", http_status=response.status_code, json_body=response.text)

    def request_api(self, request: BnplRequest) -> requests.Response:
        """
        Вызов API сервиса
        :param request: запрос ``podeli.model.BnplRequest``
        :return: ответ сервиса ``requests.Response``
        :rtype: requests.Response
        :raises: ``ValueError``: если указан метод не `GET` или `POST`
        """
        return self.__request_api_raw(method=request.method, path=request.path, data=request.message,
                                      headers=request.headers)

    def __request_api_raw(self, method: str, path: str, data: None, headers: dict = None) -> requests.Response:
        """
        Вызов API сервиса
        :param method: http метод (GET или POST)
        :param path: путь вызова
        :param data: передаваемые данные JSON
        :param headers: http заголовки
        :return: ответ сервиса ``requests.Response``
        :rtype: requests.Response
        :raises: ValueError: если указан метод не GET или POST
        """
        url = self.BaseUrl + '/' + path
        if method == "POST":
            return requests.post(url,
                                 json=data,
                                 headers=headers,
                                 auth=(self.login, self.password),
                                 cert=(self.cert_file, self.cert_key),
                                 # timeout=30,
                                 verify=self.verify_ssl,
                                 proxies=self.proxy
                                 )
        elif method == "GET":
            return requests.get(url,
                                json=data,
                                headers=headers,
                                auth=(self.login, self.password),
                                cert=(self.cert_file, self.cert_key),
                                timeout=30,
                                verify=False,
                                proxies=self.proxy
                                )
        else:
            raise ValueError(
                "http method must be `GET` or `POST`"
            )
